/*------------------------------------
 * os.c -  Fekete Krisztian
 * fekete.krisztian@freemail.hu
 *------------------------------------
 */

#include <unistd.h>             /* fork() */
#include <sys/types.h>
#include <sys/wait.h>           /* waitpid() */
#include <stdlib.h>             /* exit() */
#include "os.h"

defglob os_chdir(
     const char* path)
{
     assert(path);
     
     IFDEBUG(printf("chdir: %s", path));
     if (chdir(path)) {
          fail;
     } else {
          succeed;
     }
}

defglob os_spawn(
     const char* cmd)
{
     pid_t pid;
     
     assert(cmd);
     
     IFDEBUG(printf("spawn: %s", cmd));
     pid = fork();
     if (0 == pid) {
          execl("/bin/sh", "sh", "-c", cmd, (char*)0);
          IFDEBUG(printf("execl: /bin/sh FAILED"));
          exit(1);
     }
     if (-1 == pid) {
          fail;
     } else {
          
          succeed;
     }
}

defglob os_spawn_with_dir(
     const char* dir,
     const char* cmd)
{
     if (dir) {
          if (!os_chdir(dir)) {
               fail;
          }
     }
     
     return os_spawn(cmd);
}

defglob os_reap_zombies()
{
     int status;
     yesno reap;

     for (reap = yes; reap;) {
          switch (waitpid(-1, &status, WNOHANG)) {
          case 0:
               reap = no;
               break;
          case -1:
               /* error */
               reap = no;
               break;
          default:
               break;
          }
     }
     succeed;
}

/* end of file */
